
"""
    Aignostics Platform API reference

     Pagination is done via `page` and `page_size`. Sorting via `sort` query parameter. The `sort` query parameter can be provided multiple times. The sorting direction can be indicated via `+` (ascending) or `-` (descending) (e.g. `/v1/applications?sort=+name)`.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ApplicationRunStatus(str, Enum):
    """
    ApplicationRunStatus
    """

    """
    allowed enum values
    """
    CANCELED_SYSTEM = 'canceled_system'
    CANCELED_USER = 'canceled_user'
    COMPLETED = 'completed'
    COMPLETED_WITH_ERROR = 'completed_with_error'
    RECEIVED = 'received'
    REJECTED = 'rejected'
    RUNNING = 'running'
    SCHEDULED = 'scheduled'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ApplicationRunStatus from a JSON string"""
        return cls(json.loads(json_str))
