"""Messages for the Aignostics client."""

AUTHENTICATION_FAILED = "Authentication failed. Please check your credentials."
NOT_YET_IMPLEMENTED = "Not yet implemented."
UNKNOWN_ENDPOINT_URL = "Unknown endpoint URL. Please check the API documentation."
INVALID_REDIRECT_URI = "Invalid redirect URI. Please check the redirect URI in your application settings."
