"""Client specific constants."""

TODO_URL = "https://todo"
API_ROOT_PRODUCTION = "https://platform.aignostics.com"
AUDIENCE_PRODUCTION = "https://aignostics-platform-samia"
AUTHORIZATION_BASE_URL_PRODUCTION = "https://aignostics-platform.eu.auth0.com/authorize"
TOKEN_URL_PRODUCTION = "https://aignostics-platform.eu.auth0.com/oauth/token"  # noqa: S105
REDIRECT_URI_PRODUCTION = "http://localhost:8989/"
DEVICE_URL_PRODUCTION = "https://aignostics-platform.eu.auth0.com/oauth/device/code"
JWS_JSON_URL_PRODUCTION = "https://aignostics-platform.eu.auth0.com/.well-known/jwks.json"

API_ROOT_STAGING = "https://platform-staging.aignostics.com"
# TODO (Andreas): hhva: please fill in
AUDIENCE_STAGING = TODO_URL
AUTHORIZATION_BASE_URL_STAGING = TODO_URL
TOKEN_URL_STAGING = TODO_URL
REDIRECT_URI_STAGING = TODO_URL
DEVICE_URL_STAGING = TODO_URL
JWS_JSON_URL_STAGING = TODO_URL

API_ROOT_DEV = "https://platform-dev.aignostics.com"
AUDIENCE_DEV = "https://dev-8ouohmmrbuh2h4vu-samia"
AUTHORIZATION_BASE_URL_DEV = "https://dev-8ouohmmrbuh2h4vu.eu.auth0.com/authorize"
TOKEN_URL_DEV = "https://dev-8ouohmmrbuh2h4vu.eu.auth0.com/oauth/token"  # noqa: S105
REDIRECT_URI_DEV = "http://localhost:8989/"
DEVICE_URL_DEV = "https://dev-8ouohmmrbuh2h4vu.eu.auth0.com/oauth/device/code"
JWS_JSON_URL_DEV = "https://dev-8ouohmmrbuh2h4vu.eu.auth0.com/.well-known/jwks.json"
